/* $Id: serialport.h 358 2010-01-16 06:51:37Z oh2gve $
 * 
 * Copyright 2009, 2010 Tapio Aaltonen
 * 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SERIALPORT_H
#define SERIALPORT_H


#include <string>



/// Represents a POSIX serial port device.
class Serialport
{
public:
		
	/// Constructor.
	/**
	 * Supported baudrates: 300, 600, 1200, 2400, 4800, 9600, 19200, 38400, 57600
	 * Supported charsizes: 7, 8
	 * Supported parities: N, E, O
	 * Supported stopbit counts: 1, 2
	*/
	Serialport(std::string const& deviceFile, std::string const& lockFile, unsigned int const& baudrate, unsigned int const& charsize, char const& parity, unsigned int const& stopbits, bool const& rtscts);
	
	/// Destructor.
	~Serialport();
	
	/// Writes given data to the port device.
	/**
	 * \return True if all data was written, false in other cases.
	 */
	bool transmit(std::string const& data);
	
	/// Reads bytes from device input buffer or timeouts after 100 ms if buffer is empty.
	/**
	 * \return True in case of received data or timeout, false if error happened.
	 */
	bool receive(std::string& data);
	
	/// Check port status.
	bool operator!() const;
	
	/// Returns error message.
	std::string error() const;
	
private:
	
	/// Copy constructor not allowed.
	Serialport(Serialport const&);

	/// Assignment not allowed.
	Serialport& operator=(Serialport const&);

	
	/// File descriptor of the device.
	int fd_;
	
	/// Name of lock file.
	std::string lockFileName_;
	
	/// In case of error the message is stored here.
	std::string error_;
};

#endif // SERIALPORT_H
