/* $Id: tnc.h 358 2010-01-16 06:51:37Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009, 2010 Tapio Aaltonen
 *
 *
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef TNC_H
#define TNC_H

// Own stuff.
#include "cfg.h"
#include "serial.h"

// std stuff.
#include <string>
#include <vector>

// wx stuff.
#include <wx/string.h>
#include <wx/thread.h>


/// Implements Serial for a TNC.
class Tnc : public Serial
{
 public:
	
	/// Simply passes all arguments to the constructor of Serial.
	Tnc(wxString const& device, Cfg::ConnectionState const& state,
		 Cfg::SerialCfg portcfg, wxString const& lockdir);
	
	// There's no dynamic memory handling => no self-made destructor.

	/// This gets automatically called upon starting the thread.
	virtual void* Entry();
	
 private:

	/// Address of device is stored also here for error messaging.
	wxString const device_;
	

	/// Searches input buffer for a packet and moves it to tnc2Packet_ if found.
	bool findTnc2Packet();
	

	// Data storage
	
	/// Bytes received from device are stored first here.
	std::string inputBuffer_;
	
	/// Once a packet is parsed from a frame, it's stored here.
	std::string tnc2Packet_;

};

#endif
